package common
{
	import flash.utils.ByteArray;
	
	public class PNGData
	{
		private var head:ByteArray = new ByteArray();
		public var palette:ByteArray;
		private var tail:ByteArray = new ByteArray();
		
		public var colorlength:int;
		
		public function PNGData(bytes:ByteArray)
		{
			bytes.position = 0;
			
			var writeref:ByteArray = head;
			
			bytes.readBytes(writeref, 0, 8);	//PNG識別部
			writeref.position = 8;
			
			for(var i:int = 0; bytes.bytesAvailable > 0; i++)
			{
				var chunk:ByteArray = new ByteArray();
				
				var len:uint = bytes.readUnsignedInt();
				var type:String = bytes.readMultiByte(4, "us-ascii");
				
				chunk.writeMultiByte(type, "us-ascii");
				
				if(len != 0) {
					bytes.readBytes(chunk, chunk.length, len);
				}
				
				switch(type)
				{
				case "PLTE":	//パレット
					palette = chunk;
					bytes.position += 4;
					
					writeref = tail;
					break;
				case "IHDR":	//ヘッダ
					colorlength = 1 << chunk[4+8];
				default:
					writeref.writeUnsignedInt(chunk.length-4);
					writeref.writeBytes(chunk);
					writeref.writeUnsignedInt(bytes.readUnsignedInt());
					break;
				}
			}
			return;
		}
		
		public function getBytes():ByteArray
		{
			var bytes:ByteArray = new ByteArray();
			bytes.writeBytes(head);
			
			bytes.writeUnsignedInt(palette.length-4);
			bytes.writeBytes(palette);
			palette.position = 0;
			bytes.writeUnsignedInt(crc(palette));
			
			bytes.writeBytes(tail);
			return bytes;
		}
		
		public function getColor(num:uint):uint
		{
			if(num >= colorlength) {throw new ArgumentError();}
			
			var color:uint = 0;
			
			palette.position = 4 + num*3;
			
			color += palette.readUnsignedByte() << 16 & 0xFF0000;
			color += palette.readUnsignedByte() << 8 & 0x00FF00;
			color += palette.readUnsignedByte() & 0x0000FF;
			
			return color;
		}
		
		public function setColor(num:uint, color:uint):void
		{
			if(num >= colorlength) {throw new ArgumentError();}
			
			palette.position = 4 + num*3;
			
			palette.writeByte((color & 0xFF0000) >> 16);
			palette.writeByte((color & 0x00FF00) >> 8);
			palette.writeByte((color & 0x0000FF));
			
			return;
		}
		
		/* Table of CRCs of all 8-bit messages. */
		private static var crc_table:Array;
		
		/* Flag: has the table been computed? Initially false. */
		private static var crc_table_computed:Boolean = false;
		
		/* Make the table for a fast CRC. */
		private function make_crc_table():void
		{
			var c:uint;
			crc_table = new Array();
			
			for (var n:int = 0; n < 256; n++)
			{
				c = n;
				for (var k:int = 0; k < 8; k++)
				{
					if (c & 1) {
						c = uint(uint(0xedb88320) ^ uint(c >>> 1));
					}
					else {
						c = uint(c >>> 1);
					}
				}
				crc_table[n] = c;
			}
			crc_table_computed = true;
		}
		
		/* Update a running CRC with the bytes buf[0..len-1]--the CRC
		should be initialized to all 1's, and the transmitted value
		is the 1's complement of the final running CRC (see the
		crc() routine below)). */
		
		private function update_crc(crc:uint, buf:ByteArray):uint
		{
			if (!crc_table_computed) {
				make_crc_table();
			}
			
			var c:uint = crc;
			var v:uint;
			
			for (var n:int = 0; buf.bytesAvailable > 0; n++)
			{
				v = buf.readUnsignedByte();
				c = uint(crc_table[(c ^ v) & uint(0xff)] ^ uint(c >>> 8));
			}
			return c;
		}
		
		/* Return the CRC of the bytes buf[0..len-1]. */
		private function crc(buf:ByteArray):uint
		{
			var cc:uint = update_crc(uint(0xffffffff), buf);
			return uint(cc ^ uint(0xffffffff));
		}
	}
}