package game.obj.player
{
	import game.obj.GameObject;
	import game.obj.box.BoxBase;
	import game.obj.effect.*;
	import game.system.*;
	import game.scene.maingame.ChipInfo;
	import common.*;
	import flash.display.BitmapData;
	
	public class Player extends GameObject
	{
		public var plygrp:PlayerGraphic;
		private var frame:String;
		private var anm_run:Number = 0;
		private var mabataki:int = 45;
		
		private var vx:Number = 0;
		private var vy:Number = 0;
		private var automove_x:Number = 0;
		
		private var fly:Boolean = false;
		private var jump_powersave:int = 0;
		private var jump_allowance:int = 0;
		private var jump_rot:int = 0;
		
		public var dir:String = "RIGHT";
		public var furimuki:int = 3;
		
		private var hit_left:Boolean;
		private var hit_right:Boolean;
		private var hit_top:Boolean;
		private var hit_bottom:Boolean = true;
		
		private var stand_cloud:Boolean;
		private var cloud_pass:int;
		
		private var grab_allowance:int = 0;
		private var stock:BoxBase = null;
		private var boxpos:Object = new Object();
		
		private var guide:Guide;
		
		// --------------------------------//
		// 初期化
		// --------------------------------//
		override public function init():void
		{
			registerObject("PRIO_PLAYER", "DEPTH_PLAYER");
			plygrp = new PlayerGraphic();
			
			boxpos["STAND"] = {x:16, y:-19};
			boxpos["CROUCH"] = {x:15, y:-12};
			boxpos["RUN1"] = {x:18, y:-18};
			boxpos["RUN2"] = {x:18, y:-17};
			boxpos["RUN3"] = {x:16, y:-16};
			boxpos["RUN4"] = {x:17, y:-17};
			boxpos["FURIMUKI"] = {x:11, y:-22};
			boxpos["JUMP"] = {x:18, y:-20};
			boxpos["FALL1"] = {x:17, y:-22};
			boxpos["FALL2"] = {x:17, y:-22};
			boxpos["ROT1"] = {x:18, y:-20};
			boxpos["ROT2"] = {x:18, y:-20};
			boxpos["ROT3"] = {x:18, y:-20};
			
			frame = "STAND";
			
			guide = createObject(new Guide());
			
			func_key = new KeyWatcher();
			func_key.enabled = false;
			
			changeAction("act_stand");
			return;
		}
		
		public function act_toujo():void
		{
			if(count == 25)
			{
				playSE("SE_JUMP");
				createObject(new ToujoFlash(), 0, -64);
			}
			
			if(count == 29)
			{
				visible = true;
				fly = true;
				y -= 48;
				vy = -10;
				changeAction("act_fly");
			}
		}
		
		// --------------------------------//
		// 実行順序
		// --------------------------------//
		override public function main():void
		{
			guide.visible = false;
			
			func_key.main();
			super.main();
			
			move();
			
			if(stock != null)
			{
				if(plygrp.invert == false) {stock.x = x + boxpos[frame].x;}
				if(plygrp.invert == true) {stock.x = x - boxpos[frame].x;}
				stock.y = y + boxpos[frame].y;
			}
			if(jump_allowance > 0) {jump_allowance--;}
			if(grab_allowance > 0) {grab_allowance--;}
			
			if(mabataki < 0)
			{
				plygrp.close_eye = true;
				if(mabataki == -3) {
					mabataki = 20 + Math.random()*150;
				}
			}
			else {
				plygrp.close_eye = false;
			}
			mabataki--;
			return;
		}
		
		public function move():void
		{
			hit_left  = false;
			hit_right = false;
			hit_top    = false;
			hit_bottom = false;
			automove_x = 0;
			stand_cloud = false;
			
			if(Math.abs(vx) > Math.abs(vy))
			{
				x += vx + automove_x;
				check_merikomi_x();
				y += vy;
				check_merikomi_y();
			}
			else
			{
				y += vy;
				check_merikomi_y();
				x += vx + automove_x;
				check_merikomi_x();
			}
			return;
		}
		
		// --------------------------------//
		// 画像処理
		// --------------------------------//
		override public function draw(surface:BitmapData):void
		{
			plygrp.frame = frame;
			plygrp.grapple = (stock != null);
			
			if(func_key.isDown("UP")) {
				plygrp.muki = "U";
			}
			else if(func_key.isDown("DOWN")) {
				plygrp.muki = "D";
			}
			else {
				plygrp.muki = "F";
			}
			
			plygrp.draw(surface, getScreenX(), getScreenY());
			return;
		}
		
		// --------------------------------//
		// 地上動作
		// --------------------------------//
		public function act_stand():void
		{
			if(count == 0) {
				fly = false;
			}
			
			//操作
			if(func_key.isDown("DOWN") && stand_cloud == false) {
				changeAction("act_crouch");
				return;
			}
			else if(func_key.isDown("LEFT")) {
				vx += -2.5;
				dir = "LEFT";
			}
			else if(func_key.isDown("RIGHT")) {
				vx += 2.5;
				dir = "RIGHT";
			}
			else {
				if(vx <= -2.5) {vx += 2.5;}
				else if(vx >= 2.5) {vx -= 2.5;}
				else {vx = 0;}
				frame = "STAND";
			}
			
			if(vx >  5 + automove_x) {vx =  5 + automove_x;}
			if(vx < -5 + automove_x) {vx = -5 + automove_x;}
			
			//箱
			check_box();
			
			//アニメ
			if(vx != 0)
			{
				if     (anm_run <  5) {frame = "RUN1";}
				else if(anm_run < 15) {frame = "RUN2";}
				else if(anm_run < 25) {frame = "RUN3";}
				else if(anm_run < 30) {frame = "RUN4";}
				
				anm_run += Math.abs(vx);
				if(anm_run >= 36) {anm_run -= 36;}
			}
			check_furimuki();
			
			//ジャンプ
			if((func_key.isPress("BTN_A")) || (func_key.isDown("BTN_A") && jump_allowance > 0))
			{
				playSE("SE_JUMP");
				fly = true;
				vy = -10;
				jump_powersave = 4;
				frame = "JUMP";
				changeAction("act_fly");
				return;
			}
			
			//落下
			if(hit_bottom == false || (stand_cloud == true && func_key.isDown("DOWN")))
			{
				fly = true;
				jump_powersave = 0;
				cloud_pass = 5;
				frame = "FALL1";
				changeAction("act_fly");
				return;
			}
			return;
		}
		
		public function act_crouch():void
		{
			if(count == 0)
			{
				vx = 0;
				frame = "CROUCH";
			}
			
			check_box_crouch();
			
			if(func_key.isDown("DOWN") == false) {
				changeAction("act_stand");
				return;
			}
			
			//落下
			if(hit_bottom == false)
			{
				fly = true;
				jump_powersave = 0;
				frame = "FALL1";
				changeAction("act_fly");
				return;
			}
			return;
		}
		
		// --------------------------------//
		// 空中動作
		// --------------------------------//
		public function act_fly():void
		{
			if(count == 0)
			{
				jump_rot = 0;
			}
			
			//操作
			if(func_key.isDown("LEFT"))
			{
				if(vx > -4) {vx -= 1;}
				dir = "LEFT";
			}
			else if(func_key.isDown("RIGHT"))
			{
				if(vx < 4) {vx += 1;}
				dir = "RIGHT";
			}
			if(func_key.isDown("BTN_A")) {
				if(jump_powersave > 0)
				{
					vy = -10;
					jump_powersave--;
				}
			}
			else {
				jump_powersave = 0;
			}
			if(func_key.isPress("BTN_A"))
			{
				jump_allowance = 4;
			}
			
			if(hit_top == true && vy <= 0)
			{
				jump_powersave = 0;
				vy = 0;
			}
			
			vy += 1;
			if(vy > 10) {vy = 10;}
			
			//箱
			check_box();
			
			//アニメーション
			if(vy == -4) {jump_rot = 6;}
			
			if(jump_rot > 0)
			{
				if(jump_rot > 4)      {frame = "ROT1";}
				else if(jump_rot > 2) {frame = "ROT2";}
				                 else {frame = "ROT3";}
				jump_rot--;
			}
			else if(vy < 0) {
				frame = "JUMP";
			}
			else {
				if((count % 4) < 2) {frame = "FALL1";}
				               else {frame = "FALL2";}
			}
			check_furimuki();
			
			//着地
			if(hit_bottom == true && vy > 0)
			{
				playSE("SE_CHAKUCHI");
				
				createObject(new Smoke(-5, 0), 0, -3);
				createObject(new Smoke(5, 0), 0, -3);
				
				vy = 0;
				changeAction("act_stand");
			}
		}
		
		// --------------------------------//
		// 当たり判定
		// --------------------------------//
		public function check_merikomi():void
		{
			if(Math.abs(vx) > Math.abs(vy))
			{
				check_merikomi_x();
				check_merikomi_y();
			}
			else
			{
				check_merikomi_y();
				check_merikomi_x();
			}
		}
		
		public function check_merikomi_x():void
		{
			//箱判定
			var boxlist:List = scene.func_obj.searchPriority("PRIO_BOX");
			var boxhit_l:Boolean = false;
			var boxhit_r:Boolean = false;
			var boxhit_lmodosipos:Number = Number.NEGATIVE_INFINITY;
			var boxhit_rmodosipos:Number = Number.POSITIVE_INFINITY;
			
			boxlist.forEach(function(box:BoxBase):void
			{
				if(box.uncollidable_allowance > 0) {return;}
				
				if(box.hitTest(x-9, y-1) || box.hitTest(x-9, y-28))
				{
					boxhit_lmodosipos = Math.max(boxhit_lmodosipos, box.x + 24-(12-10));
					boxhit_l = true;
				}
				if(box.hitTest(x+9, y-1) || box.hitTest(x+9, y-28))
				{
					boxhit_rmodosipos = Math.min(boxhit_rmodosipos, box.x - 24+(12-10));
					boxhit_r = true;
				}
			});
			
			if(boxhit_l == true && boxhit_r == true)
			{
				boxhit_l = false;
				boxhit_r = false;
			}
			
			if(boxhit_l == true && vx <= 0)
			{
				x = boxhit_lmodosipos;
				vx = 0;
				hit_left = true;
			}
			if(boxhit_r == true && vx >= 0)
			{
				x = boxhit_rmodosipos;
				vx = 0;
				hit_right = true;
			}
			
			//地形判定
			var l1:ChipInfo = getChipInfo(-10, -1);
			var l2:ChipInfo = getChipInfo(-10, -28);
			var r1:ChipInfo = getChipInfo(10, -1);
			var r2:ChipInfo = getChipInfo(10, -28);
			
			var wallhit_l:Boolean = false;
			var wallhit_r:Boolean = false;
			
			if(l1.wall || l2.wall) {wallhit_l = true;}
			if(r1.wall || r2.wall) {wallhit_r = true;}
			
			if(wallhit_l == true && wallhit_r == true)
			{
				wallhit_l = false;
				wallhit_r = false;
			}
			
			if(wallhit_l  == true && vx <= 0)
			{
				if(x < 10) {x = 10;}
				else {x += 24-((x-10)%24);}
				vx = 0;
				hit_left = true;
			}
			if(wallhit_r == true && vx >= 0)
			{
				x -= (x+10) % 24;
				vx = 0;
				hit_right = true;
			}
			
			return;
		}
		
		public function check_merikomi_y():void
		{
			//箱判定
			var boxlist:List = scene.func_obj.searchPriority("PRIO_BOX");
			var boxhit_t:Boolean = false;
			var boxhit_b:Boolean = false;
			var boxhit_tmodosipos:Number = Number.NEGATIVE_INFINITY;
			var boxhit_bmodosipos:Number = Number.POSITIVE_INFINITY;
			
			boxlist.forEach(function(box:BoxBase):void
			{
				if(box.uncollidable_allowance > 0) {return;}
				if(box.isActive() == true) {return;}
				
				if(box.hitTest(x, y-38))
				{
					boxhit_tmodosipos = Math.max(boxhit_tmodosipos, box.y + 24-(12-38));
					boxhit_t = true;
				}
				if(box.hitTest(x-9, y+1) || box.hitTest(x+9, y+1))
				{
					boxhit_bmodosipos = Math.min(boxhit_bmodosipos, box.y - 24+12);
					boxhit_b = true;
					
					automove_x = box.automove_x;
				}
			});
			
			if(boxhit_t == true && boxhit_b == true)
			{
				boxhit_t = false;
			}
			
			if(boxhit_t == true && vy <= 0)
			{
				y = boxhit_tmodosipos;
				vy = 0;
				hit_top = true;
			}
			if(boxhit_b == true && vy >= 0)
			{
				y = boxhit_bmodosipos;
				vy = 0;
				hit_bottom = true;
			}
			
			//地形判定
			var t:ChipInfo = getChipInfo(0, -38);
			var b1:ChipInfo = getChipInfo(-9, 0);
			var b2:ChipInfo = getChipInfo(9, 0);
			
			var wallhit_t:Boolean = false;
			var wallhit_b:Boolean = false;
			
			if(t.wall) {wallhit_t = true;}
			if(b1.wall || b2.wall) {wallhit_b = true;}
			
			if(cloud_pass == 0) {
				if((b1.cloud || b2.cloud) && (b1.wall == false && b2.wall == false) && vy >= 0)
				{
					stand_cloud = true;
					wallhit_b = true;
				}
			}
			else {
				cloud_pass--;
			}
			
			if(wallhit_t == true && wallhit_b == true)
			{
				wallhit_t = false;
			}
			
			if(wallhit_t == true && vy <= 0)
			{
				y += 24-((y-38)%24);
				vy = 0;
				hit_top = true;
			}
			if(wallhit_b == true && vy >= 0)
			{
				y -= y % 24; vy = 0;
				automove_x = b1.automove + b2.automove;
				hit_bottom = true;
			}
			
			return;
		}
		
		// --------------------------------//
		// ふりむき処理
		// --------------------------------//
		public function check_furimuki():void
		{
			if(furimuki > 0) {
				plygrp.invert = false;
			}
			else {
				plygrp.invert = true;
			}
			
			if(dir == "LEFT") {
				if(furimuki > -2) {
					furimuki--;
				}
			}
			else {
				if(furimuki < 3) {
					furimuki++;
				}
			}
			
			if(jump_rot > 0 && jump_rot < 6) {return;}
			
			if(furimuki > -2 && furimuki < 3) {
				frame = "FURIMUKI";
			}
			
			return;
		}
		
		// --------------------------------//
		// 箱の処理
		// --------------------------------//
		public function getBox(x1:Number, y1:Number, x2:Number, y2:Number):Array
		{
			if(dir == "LEFT")
			{
				x1 = -x1;
				x2 = -x2;
				
				//値の入れ替え
				var t:Number;
				t = x1;
				x1 = x2;
				x2 = t;
			}
			
			x1 += x;
			y1 += y;
			x2 += x;
			y2 += y;
			
			var objlist:Array = new Array();
			var boxlist:List = scene.func_obj.searchPriority("PRIO_BOX");
			
			boxlist.forEach(function(box:BoxBase):void
			{
				if(x1 <= box.x && y1 <= box.y && x2 >= box.x && y2 >= box.y)
				{
					objlist.push(box);
				}
			});
			
			return objlist;
		}
		
		public function check_box():void
		{
			if(func_key.isPress("BTN_B")) {
				grab_allowance = 3;
			}
			
			if(stock == null)
			{
				//つかみ
				var box:Array = getBox(8, -32, 32, -8);
				
				if(box.length != 0)
				{
					guide.visible = true;
					guide.x = box[0].x;
					guide.y = box[0].y;
					
					if(func_key.isPress("BTN_B") || (func_key.isDown("BTN_B") && grab_allowance > 0))
					{
						playSE("SE_GRAB");
						stock = box[0];
						stock.grab();
					}
				}
			}
			else
			{
				//投げ
				if(func_key.isPress("BTN_B"))
				{
					playSE("SE_THROW");
					if(dir == "LEFT")  {createObject(new Throw(vx/2, vy/2), -16, -20);}
					if(dir == "RIGHT") {createObject(new Throw(vx/2, vy/2), 16, -20);}
					
					grab_allowance = 0;
					
					if(func_key.isDown("UP"))
					{
						if(func_key.isDown(dir))
						{
							if(dir == "LEFT")  {stock.vx = -6;}
							if(dir == "RIGHT") {stock.vx = 6;}
							stock.vy = -11;
						}
						else {
							stock.vx = vx/2;
							stock.vy = -14;
						}
					}
					else if(func_key.isDown("DOWN"))
					{
						if(func_key.isDown(dir))
						{
							if(dir == "LEFT")  {stock.vx = -7;}
							if(dir == "RIGHT") {stock.vx = 7;}
							stock.vy = 4;
						}
						else {
							stock.vx = vx/2;
							stock.vy = 11;
						}
					}
					else
					{
						if(dir == "LEFT")  {stock.vx = -8;}
						if(dir == "RIGHT") {stock.vx = 8;}
					}
					if(dir == "LEFT")  {stock.x = x-16;}
					if(dir == "RIGHT") {stock.x = x+16;}
					stock.y = y-20;
					
					stock.ungrab();
					stock.uncollidable_allowance = 5;
					stock = null;
				}
			}
		}
		
		public function check_box_crouch():void
		{
			if(func_key.isPress("BTN_B")) {
				grab_allowance = 3;
			}
			
			if(stock == null)
			{
				//つかみ
				var box:Array = getBox(-10, 0, 14, 24);
				
				if(box.length != 0)
				{
					guide.visible = true;
					guide.x = box[0].x;
					guide.y = box[0].y;
					
					if(func_key.isPress("BTN_B") || (func_key.isDown("BTN_B") && grab_allowance > 0))
					{
						playSE("SE_GRAB");
						stock = box[0];
						stock.grab();
					}
				}
			}
			else
			{
				//置く
				if(func_key.isPress("BTN_B"))
				{
					grab_allowance = 0;
					if(dir == "LEFT")  {stock.x -= 4;}
					if(dir == "RIGHT") {stock.x += 4;}
					stock.y -= 2;
					
					stock.ungrab();
					stock.uncollidable_allowance = 2;
					stock = null;
				}
			}
		}
	}
}