﻿package game.scene
{
	import game.system.*;
	import game.obj.system.*;
	import common.*;
	import flash.utils.ByteArray;
	
	public class MenuScene extends SceneBase
	{
		private var mainmenu:ListMenu = new ListMenu();
		private var option:ListMenu = new ListMenu();
		private var replay:ListMenu = new ListMenu();
		
		private var bg:BG;
		private var selectedcourse:int;
		private var course:Object = new Object();
		
		private var gamedata:GameData = new GameData();
		
		override public function init():void
		{
			func_bgm.play("BGM_MENU");
			createObject(new FadeIn(45));
			
			mainmenu.addItem("START");
			mainmenu.addItem("STAGE SELECT");
			mainmenu.addItem("RANKING");
			mainmenu.addItem("REPLAY");
			mainmenu.addItem("OPTION");
			mainmenu.linespace = 8;
			
			var bgmlist:Array = new Array
			(
				"STOP",
				"COURSE1",
				"COURSE2",
				"ENTRY",
				"MENU",
				"STANDBY"
			);
			var selist:Array = new Array
			(
				"STOP",
				"JUMP",
				"CHAKUCHI",
				"GRAB",
				"THROW",
				"BOXCHAKUCHI",
				"SUBERI",
				"STAGECLEAR",
				"ALLCLEAR",
				"TIMEOUT",
				"SELECT",
				"ENTER",
				"CANCEL",
				"PAUSE",
				"START"
			);
			var palettelist:Array = new Array
			(
				"NORMAL",
				"NIGHT",
				"PASTEL",
				"VECTOR-LIKE",
				"RANDOM"
			);
			
			option.addItem("PALETTE", false, palettelist);
			option.addItem("FPS", false, new Array("OFF", "ON"));
			option.addItem("BGM TEST", false, bgmlist);
			option.addItem("SE TEST", false, selist);
			option.width = 160;
			option.linespace = 8;
			
			replay.addItem("COURSE 1");
			replay.addItem("COURSE 2");
			
			for(var i:int = 1; i <= 6; i++) {
				replay.addItem("STAGE 1-"+i);
			}
			for(var i:int = 1; i <= 8; i++) {
				replay.addItem("STAGE 2-"+i);
			}
			for each(var item:MenuItem in replay.itemlist)
			{
				item.font_normal = "8px_trans";
				item.font_select = "8px";
			}
			replay.getItem("COURSE 2").linespace = 4;
			replay.getItem("STAGE 1-6").linespace = 4;
			replay.linespace = 2;
			
			bg = createObject(new BG());
			
			var mes:Message = createObject(new Message("8px", "CURSOR-SELECT  Z-DECIDE  SHIFT-RETURN"));
			mes.x = 160 - mes.str.length*4;
			mes.y = 8;
			
			changeAction("act_mainmenu");
		}
		
		// --------------------------------//
		// メインメニュー
		// --------------------------------//
		public function act_mainmenu():void
		{
			if(count == 0)
			{
				mainmenu.x = 88;
				mainmenu.y = 112;
				createObject(mainmenu);
			}
			
			if(mainmenu.isDecided())
			{
				changeAction("act_mainmenu_decide");
			}
		}
		
		public function act_mainmenu_decide():void
		{
			if(count == 15)
			{
				switch(mainmenu.currentitem.label)
				{
				case "START":
					changeAction("act_courseselect");
					break;
				case "STAGE SELECT":
					changeAction("act_gotostageselect");
					break;
				case "RANKING":
					changeAction("act_gotoranking");
					break;
				case "REPLAY":
					changeAction("act_replay");
					break;
				case "OPTION":
					changeAction("act_option");
					break;
				}
				return;
			}
		}
		
		// --------------------------------//
		// コースセレクト
		// --------------------------------//
		public function act_courseselect():void
		{
			if(count == 0)
			{
				selectedcourse = 1;
				
				course[1] = createObject(new Course(1));
				course[2] = createObject(new Course(2));
			}
			
			if(func_key.isPress("BTN_B"))
			{
				playSE("SE_CANCEL");
				course[1].vanish();
				course[2].vanish();
				changeAction("act_mainmenu");
				return;
			}
			
			if(func_key.isPress("BTN_A"))
			{
				playSE("SE_ENTER");
				gamedata.mode = "SEQUENTIAL";
				gamedata.course = selectedcourse;
				changeAction("act_course_decide");
				return;
			}
			
			if(func_key.isPress("UP", true) || func_key.isPress("DOWN", true))
			{
				playSE("SE_SELECT");
				
				switch(selectedcourse)
				{
				case 1: selectedcourse = 2; break;
				case 2: selectedcourse = 1; break;
				}
			}
			
			if(selectedcourse == 1)
			{
				course[1].selected = true;
				course[2].selected = false;
			}
			if(selectedcourse == 2)
			{
				course[2].selected = true;
				course[1].selected = false;
			}
		}
		
		public function act_course_decide():void
		{
			if(count == 0) {
				func_bgm.fadeout();
				course[selectedcourse].changeAction("act_decide");
			}
			
			if(count == 18)
			{
				course[1].vanish();
				course[2].vanish();
				
				changeAction("act_gotomaingame");
			}
		}
		
		// --------------------------------//
		// リプレイ
		// --------------------------------//
		private var txt_records:Array;
		public function act_replay():void
		{
			if(count == 0)
			{
				replay.x = 80;
				replay.y = 48;
				replay.selectedindex = 0;
				createObject(replay);
				
				var y:int = replay.y;
				txt_records = new Array();
				
				for(var i:int = 0; i < replay.itemlist.length; i++)
				{
					var mes:Message = new Message("SYSTEM");
					var temp:Array = replay.itemlist[i].label.split(/[ -]/);
					
					if(temp[0] == "COURSE") {mes.str = PlayData.replays.getScore(temp[1]);}
					if(temp[0] == "STAGE") {mes.str = PlayData.replays.getTime(temp[1], temp[2]);}
					
					if(mes.str == "no data") {
						replay.itemlist[i].button = false;
					}
					else {
						replay.itemlist[i].button = true;
					}
					
					mes.x = (320-80)-(mes.str.length*6);
					mes.y = y;
					y += 10 + replay.itemlist[i].linespace;
					
					txt_records.push(createObject(mes));
				}
			}
			
			if(func_key.isPress("BTN_B"))
			{
				playSE("SE_CANCEL");
				replay.vanish();
				for each(var mes:Message in txt_records) {
					mes.vanish();
				}
				changeAction("act_mainmenu");
				return;
			}
			
			if(replay.isDecided())
			{
				for each(var mes:Message in txt_records) {
					mes.vanish();
				}
				changeAction("act_replay_decide");
			}
		}
		
		public function act_replay_decide():void
		{
			if(count == 15)
			{
				var temp:Array = replay.currentitem.label.split(/[ -]/);
				
				if(temp[0] == "COURSE")
				{
					gamedata.mode = "REPLAY_SEQUENTIAL";
					gamedata.sequence = PlayData.replays.data.course[temp[1]].sequence;
					gamedata.course = temp[1];
				}
				if(temp[0] == "STAGE")
				{
					gamedata.mode = "REPLAY_SINGLE";
					gamedata.replay = PlayData.replays.data.stage[temp[1]][temp[2]].replay;
					gamedata.course = temp[1];
					gamedata.stage = temp[2];
				}
				
				changeAction("act_gotomaingame");
			}
		}
		
		// --------------------------------//
		// コンフィグ
		// --------------------------------//
		private var prepalette:String;
		private var nowchangeing:Boolean = false;
		
		public function act_option():void
		{
			if(nowchangeing == true)
			{
				if(GraphicConverter.isComplete())
				{
					bg.reload();
					nowchangeing = false;
				}
				return;
			}
			
			if(count == 0)
			{
				option.x = 80;
				option.y = 128;
				option.selectedindex = 0;
				
				option.getItem("PALETTE").currentdata = PlayData.option.data.palette;
				option.getItem("FPS").currentdata = PlayData.option.data.fps;
				
				prepalette = PlayData.option.data.palette;
				
				createObject(option);
			}
			
			if(option.currentitem != null)
			{
				var data:String = option.currentitem.currentdata;
				
				switch(option.currentitem.label)
				{
				case "PALETTE":
					if(data != prepalette)
					{
						var palette:ByteArray;
						
						switch(data)
						{
						case "NORMAL":palette = new PNGData(Database.getItem("PNG_PAL1")).palette; break;
						case "NIGHT":palette = new PNGData(Database.getItem("PNG_PAL2")).palette; break;
						case "PASTEL":palette = new PNGData(Database.getItem("PNG_PAL3")).palette; break;
						case "VECTOR-LIKE":palette = new PNGData(Database.getItem("PNG_PAL4")).palette; break;
						case "RANDOM":
							palette = new ByteArray();
							palette.writeMultiByte("PLTE", "us-ascii");
							for(var i:int = 0; i < 768; i++) {
								palette.writeByte(Math.random()*0x100);
							}
							break;
						}
						GraphicConverter.convert(new XML(Database.getItem("XML_GRAPHIC")), palette);
						
						prepalette = data;
						nowchangeing = true;
						return;
					}
					break;
				case "FPS":
					if(data == "OFF") {AppInfo.hideFPS();}
					if(data == "ON") {AppInfo.viewFPS();}
					break;
				case "BGM TEST":
					if(func_key.isPress("BTN_A"))
					{
						func_bgm.stop();
						if(data != "STOP") {
							func_bgm.play("BGM_"+data);
						}
					}
					break;
				case "SE TEST":
					if(func_key.isPress("BTN_A"))
					{
						if(data == "STOP") {stopSE();}
						else {playSE("SE_"+data);}
					}
					break;
				}
			}
			
			if(func_key.isPress("BTN_B"))
			{
				PlayData.option.data.palette = option.getItem("PALETTE").currentdata;
				PlayData.option.data.fps = option.getItem("FPS").currentdata;
				PlayData.option.saveData();
				
				func_bgm.play("BGM_MENU");
				
				playSE("SE_CANCEL");
				option.vanish();
				changeAction("act_mainmenu");
				return;
			}
		}
		
		// --------------------------------//
		// シーン変更
		// --------------------------------//
		public function act_gotomaingame():void
		{
			if(count == 0) {
				createObject(new FadeOut());
			}
			
			if(count == 28) {
				changeScene(new MainGameScene(gamedata));
			}
			return;
		}
		
		public function act_gotostageselect():void
		{
			if(count == 0) {
				createObject(new FadeOut());
			}
			
			if(count == 28)
			{
				gamedata.mode = "STAGESELECT";
				changeScene(new MainGameScene(gamedata));
			}
			return;
		}
		
		public function act_gotoranking():void
		{
			if(count == 0) {
				createObject(new FadeOut());
			}
			
			if(count == 28) {
				changeScene(new ResultScene());
			}
			return;
		}
	}
}

import game.obj.GameObject;
import game.system.Graphic;
import game.system.Database;
import flash.geom.Rectangle;
import flash.geom.Point;
import flash.display.BitmapData;
import flash.geom.Matrix;
import flash.display.BlendMode;
import game.obj.system.Tile;
import flash.geom.ColorTransform;
import flash.display.Sprite;
import flash.media.SoundMixer;
import flash.utils.ByteArray;

// --------------------------------//
// コースセレクトの画像
// --------------------------------//
class Course extends GameObject
{
	private var course:int;
	public var selected:Boolean;
	
	public function Course(num:int)
	{
		course = num;
	}
	
	override public function init():void
	{
		registerObject("PRIO_SYSTEM", "DEPTH_MESSAGE");
		grp = new Graphic("GRP_COURSE");
		
		if(course == 1)
		{
			x = 32;
			y = 16+8;
			grp.frame = "COURSE_1";
		}
		if(course == 2)
		{
			x = 320-216 - 32;
			y = 240-96 - 16;
			grp.frame = "COURSE_2";
		}
		
		changeAction("act_main");
		return;
	}
	
	public function act_main():void
	{
		if(count == 0) {
			grp.type = "FLASH";
			return;
		}
		
		if(selected == true) {
			grp.type = "NORMAL";
		}
		else {
			grp.type = "DARK";
		}
	}
	
	public function act_decide():void
	{
		if(count % 6 < 3) {
			grp.type = "FLASH";
		}
		else {
			grp.type = "NORMAL";
		}
	}
}

// --------------------------------//
// 背景
// --------------------------------//
class BG extends GameObject
{
	private var objlist:Array = new Array();
	
	private var sprite:Sprite = new Sprite();
	private var matrix:Matrix = new Matrix();
	
	override public function init():void
	{
		create();
		
		//セロファン
		sprite.graphics.beginFill(0x202020);
		sprite.graphics.drawRect(160-96, 0, 192, 240);
		
		//このオブジェクトが一番手前
		registerObject("PRIO_BG", "DEPTH_EFFECT");
	}
	
	private function create():void
	{
		var temp:BitmapData = Database.getItem("BMP_MENUBG");
		
		//箱
		var tile:BitmapData = new BitmapData(48, 48, false, 0);
		tile.copyPixels(temp, new Rectangle(0, 0, 48, 48), new Point(0, 0));
		
		var bg:Tile = createObject(new Tile(tile));
		objlist.push(bg);
		bg.x = -8;
		bg.vy = -1/4;
		
		//文字ベルト
		var tile:BitmapData = new BitmapData(120, 24, true, 0);
		tile.copyPixels(temp, new Rectangle(48, 0, 120, 24), new Point(0, 0));
		
		var bg:Tile = createObject(new Tile(tile));
		objlist.push(bg);
		bg.repeat_y = false;
		bg.y = 120-tile.height*1.5+0*tile.height;
		bg.vx = -1/2;
		
		var bg:Tile = createObject(new Tile(tile));
		objlist.push(bg);
		bg.repeat_y = false;
		bg.y = 120-tile.height*1.5+1*tile.height;
		bg.vx = 1/2;
		
		var bg:Tile = createObject(new Tile(tile));
		objlist.push(bg);
		bg.repeat_y = false;
		bg.y = 120-tile.height*1.5+2*tile.height;
		bg.vx = -1/2;
		
		//まる
		var tile:BitmapData = new BitmapData(208, 208, true, 0);
		tile.copyPixels(temp, new Rectangle(0, 48, 208, 208), new Point(0, 0));
		
		var bg:Tile = createObject(new Tile(tile));
		objlist.push(bg);
		bg.vx = 1/3;
		bg.vy = 1/2;
		
		var bg:Tile = createObject(new Tile(tile));
		objlist.push(bg);
		bg.y = tile.height/2;
		bg.vx = -1/3;
		bg.vy = 1/2;
	}
	
	public function reload():void
	{
		for each(var obj:* in objlist)
		{
			obj.vanish();
		}
		objlist = new Array();
		create();
	}
	
	override public function draw(surface:BitmapData):void
	{
		surface.draw(sprite, matrix, null, BlendMode.OVERLAY);
	}
}
