package game.system
{
	import flash.display.BitmapData;
	import flash.geom.Rectangle;
	import common.DisplayImage;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import flash.utils.ByteArray;
	import common.PNGData;
	import flash.utils.setInterval;
	import flash.utils.clearInterval;
	import flash.display.Loader;
	
	public class GraphicConverter
	{
		private static var loaders:Object = new Object();
		private static var id:uint;
		private static var iscomplete:Boolean = false;
		private static var nowloading:Boolean = false;
		
		public static function convert(xml:XML, palette:ByteArray = null):void
		{
			if(nowloading == true) {return;}
			
			var xmllist:XMLList = xml.convert.children();
			
			for(var i:int = 0; i < xmllist.length(); i++)
			{
				var elem:XML = xmllist[i];
				
				switch(elem.localName())
				{
				case "img":
					var temp:PNGData = new PNGData(Database.getItem(elem.@src));
					if(palette != null) {
						temp.palette = palette;
					}
					loaders[elem.@name] = new Loader();
					loaders[elem.@name].loadBytes(temp.getBytes());
					break;
				}
			}
			
			iscomplete = false;
			nowloading = true;
			id = setInterval(complete, 100, xml);
			return;
		}
		
		private static function complete(xml:XML):void
		{
			for each(var loader:Loader in loaders)
			{
				if(loader.content == null) {return;}
			}
			
			for(var name:String in loaders)
			{
				Database.addItem(name, loaders[name].content["bitmapData"]);
			}
			
			var xmllist:XMLList = xml.divide.children();
			
			for(var i:int = 0; i < xmllist.length(); i++)
			{
				var elem:XML = xmllist[i];
				
				switch(elem.localName())
				{
				case "img":
					Database.addItem(elem.@name, GraphicConverter.xmlToGraphic(elem));
					break;
				}
			}
			
			iscomplete = true;
			nowloading = false;
			clearInterval(id);
		}
		
		public static function isComplete():Boolean
		{
			return iscomplete;
		}
		
		// --------------------------------//
		// 画像切り出し
		// --------------------------------//
		private static var grp:GraphicData;
		private static var bmp:BitmapData;
		
		public static function xmlToGraphic(xml:XML):GraphicData
		{
			grp = new GraphicData();
			bmp = Database.getItem(xml.@src);
			
			var xmllist:XMLList = xml.children();
			
			for(var i:int = 0; i < xmllist.length(); i++)
			{
				var elem:XML = xmllist[i];
				
				var x:int = elem.@x;
				var y:int = elem.@y;
				var width:int = elem.@width;
				var height:int = elem.@height;
				
				if(elem.@cx != undefined) {
					var cx:int = elem.@cx;
				}
				else {
					var cx:int = width/2;
				}
				if(elem.@cy != undefined) {
					var cy:int = elem.@cy;
				}
				else {
					var cy:int = height/2;
				}
				
				switch(elem.localName())
				{
				case "div":
					addFrame(elem.@name, new Rectangle(x, y, width, height), cx, cy);
					break;
				case "hdiv":
					divide("HORIZONTAL", elem.children(), x, y, width, height, cx, cy);
					break;
				case "vdiv":
					divide("VERTICAL", elem.children(), x, y, width, height, cx, cy);
					break;
				}
			}
			if(xml.variation != undefined) {
				addVariation(xml.variation.children());
			}
			return grp;
		}
		
		private static function divide(type:String, xmllist:XMLList, x:int, y:int, width:int, height:int, cx:int, cy:int):void
		{
			for(var i:int = 0; i < xmllist.length(); i++)
			{
				var elem:XML = xmllist[i];
				
				switch(elem.localName())
				{
				case "frame":
					if(elem.@name != undefined) {
						addFrame(elem.@name, new Rectangle(x, y, width, height), cx, cy);
					}
					else {
						addFrame(i.toString(), new Rectangle(x, y, width, height), cx, cy);
					}
					break;
				}
				
				switch(type)
				{
				case "HORIZONTAL":
					x += width;
					break;
				case "VERTICAL":
					y += height;
					break;
				}
			}
		}
		
		private static function addFrame(name:String, rect:Rectangle, cx:int, cy:int):void
		{
			grp.addFrame(name, bmp, rect, cx, cy);
		}
		
		private static function addVariation(xmllist:XMLList):void
		{
			for(var i:int = 0; i < xmllist.length(); i++)
			{
				var elem:XML = xmllist[i];
				
				var name:String = elem.@name;
				var args:Array = new Array();
				
				switch(elem.localName())
				{
				case "color":
					if(elem.@ra != undefined) {args.push(elem.@ra);} else {args.push(1);}
					if(elem.@ga != undefined) {args.push(elem.@ga);} else {args.push(1);}
					if(elem.@ba != undefined) {args.push(elem.@ba);} else {args.push(1);}
					if(elem.@aa != undefined) {args.push(elem.@aa);} else {args.push(1);}
					if(elem.@rb != undefined) {args.push(elem.@rb);} else {args.push(0);}
					if(elem.@gb != undefined) {args.push(elem.@gb);} else {args.push(0);}
					if(elem.@bb != undefined) {args.push(elem.@bb);} else {args.push(0);}
					if(elem.@ab != undefined) {args.push(elem.@ab);} else {args.push(0);}
					
					grp.addVariation(name, function(img:DisplayImage,
					  ra:Number, ga:Number, ba:Number, aa:Number,
					  rb:Number, gb:Number, bb:Number, ab:Number):DisplayImage
					{
						var result:DisplayImage = new DisplayImage(img.bmp.clone(), img.cx, img.cy);
						result.bmp.colorTransform(img.rect, new ColorTransform(ra, ga, ba, aa, rb, gb, bb, ab));
						return result;
					}
					, args);
					break;
				case "invert":
					grp.addVariation("INVERT", function(img:DisplayImage):DisplayImage
					{
						var temp:BitmapData = new BitmapData(img.rect.width, img.rect.height, true, 0);
						
						var matrix:Matrix = new Matrix();
						matrix.scale(-1.0, 1.0);
						matrix.translate(img.rect.width, 0);
						
						temp.draw(img.bmp, matrix);
						return new DisplayImage(temp, img.rect.width - img.cx, img.cy);
					}
					, null);
					break;
				}
			}
		}
	}
}