package game.system
{
	import common.Key;
	import flash.display.Sprite;
	import flash.utils.ByteArray;
	
	public class KeyWatcher
	{
		private static var watchlist:Object = new Object();
		private static var repeat_wait:uint = 22;
		private static var repeat_rate:uint = 3;
		
		private var keylist:Object = new Object();
		public var enabled:Boolean = true;
		
		private var replay:Array = new Array();
		private var log:Array = new Array();
		
		private const REPLAY_LIMIT:int = 45*90;	//記録は１分半まで
		
		public function KeyWatcher():void
		{
			for(var name:String in watchlist)
			{
				keylist[name] = new Object();
				keylist[name].down = false;
				keylist[name].press = false;
				keylist[name].release = false;
				keylist[name].repeat = false;
				keylist[name].repeatcount = 0;
			}
			return;
		}
		
		// --------------------------------//
		// 登録
		// --------------------------------//
		public static function watch(name:String, ...keycodes):void
		{
			watchlist[name] = keycodes;
			return;
		}
		
		public static function unwatch(name:String):void
		{
			delete watchlist[name];
			return;
		}
		
		public static function unwatchAll():void
		{
			watchlist = new Object();
			return;
		}
		
		// --------------------------------//
		// キー取得
		// --------------------------------//
		public function isDown(name:String):Boolean
		{
			if(keylist[name] is Object == false) {return false;}
			
			return keylist[name].down;
		}
		
		public function isPress(name:String, getrepeat:Boolean = false):Boolean
		{
			if(keylist[name] is Object == false) {return false;}
			
			if(getrepeat == false) {
				return keylist[name].press;
			}
			else {
				return keylist[name].press || keylist[name].repeat;
			}
		}
		
		public function isRelease(name:String):Boolean
		{
			if(keylist[name] is Object == false) {return false;}
			
			return keylist[name].release;
		}
		
		// --------------------------------//
		// リプレイ関連
		// --------------------------------//
		public function getReplay():ByteArray
		{
			if(log.length < REPLAY_LIMIT) {
				return ReplayEncoder.encode(log);
			}
			return null;
		}
		
		public function setReplay(replay:ByteArray):void
		{
			this.replay = ReplayEncoder.decode(replay);
			return;
		}
		
		// --------------------------------//
		// 監視メイン
		// --------------------------------//
		public function main():void
		{
			//入力情報の設定
			var input:Object = new Object();
			
			if(enabled == true)
			{
				if(replay.length > 0) {
					input = replay.shift();
				}
				else {
					for(var name:String in watchlist)
					{
						for each(var keycode:int in watchlist[name])
						{
							if(Key.isDown(keycode) == true)
							{
								input[name] = true;
								break;
							}
						}
					}
					if(log.length < REPLAY_LIMIT) {
						log.push(input);
					}
				}
			}
			else {
				for each(var keycode:int in watchlist[name])
				{
					input[name] = false;
				}
			}
			
			for(var name:String in keylist)
			{
				var key:Object = keylist[name];
				
				if(input[name] == true)
				{
					//初押し
					if(key.down == false)
					{
						key.press = true;
						key.repeat = true;
					}
					else
					{
						key.press = false;
						key.repeat = false;
					}
					key.down = true;
					key.release = false;
					
					key.repeatcount++;
					if(key.repeatcount == repeat_wait)
					{
						key.repeatcount -= repeat_rate;
						key.repeat = true;
					}
				}
				else
				{
					//離した
					if(key.down == true) {
						key.release = true;
					}
					else {
						key.release = false;
					}
					key.down = false;
					key.press = false;
					
					key.repeatcount = 0;
					key.repeat = false;
				}
			}
			return;
		}
	}
}