package game.system
{
	import flash.display.BitmapData;
	import common.List;
	import game.system.ObjBase;
	
	public class ObjectManager
	{
		private static var priolist:Object = new Object();
		private static var depthlist:Object = new Object();
		private static var priolength:int = 0;
		private static var depthlength:int = 0;
		
		private var objlist:Array = new Array();
		private var grplist:Array = new Array();
		private var deletelist:Array = new Array();
		
		private var lockflag:Boolean = false;
		
		// --------------------------------//
		// 初期化
		// --------------------------------//
		public function ObjectManager()
		{
			for each(var prio:String in priolist) {
				objlist.push(new List());
			}
			
			for each(var depth:String in depthlist) {
				grplist.push(new List());
			}
			
			return;
		}
		
		// --------------------------------//
		// プライオリティ追加
		// --------------------------------//
		public static function addPriority(prio:String):void
		{
			priolist[prio] = priolength;
			priolength++;
			return;
		}
		
		// --------------------------------//
		// 深度追加
		// --------------------------------//
		public static function addDepth(depth:String):void
		{
			depthlist[depth] = depthlength;
			depthlength++;
			return;
		}
		
		// --------------------------------//
		// 登録
		// --------------------------------//
		public function register(obj:ObjBase, prio:String, depth:String):void
		{
			objlist[priolist[prio]].push(obj);
			grplist[depthlist[depth]].push(obj);
			
			obj.priority = prio;
			obj.depth = depth;
			
			obj.visible = true;
			obj.deleteflag = false;
			return;
		}
		
		public function changeDepth(obj:ObjBase, newdepth:String):void
		{
			grplist[depthlist[obj.depth]].erase(obj);
			grplist[depthlist[newdepth]].push(obj);
			
			obj.depth = newdepth;
			return;
		}
		
		// --------------------------------//
		// 削除
		// --------------------------------//
		public function removeMovieClip(obj:ObjBase):void
		{
			//update()でまとめて消される
			obj.deleteflag = true;
			deletelist.push(obj);
			return;
		}
		
		// --------------------------------//
		// 指定プライオリティを全削除
		// --------------------------------//
		public function removePriority(prio:String):void
		{
			var list:List = this.searchPriority(prio);
			list.forEach(function(obj:ObjBase):void
			{
				removeMovieClip(obj);
			});
			
			return;
		}
		
		// --------------------------------//
		// 更新
		// --------------------------------//
		public function update():void
		{
			for each(var obj:ObjBase in deletelist)
			{
				obj.erase();	//デストラクタ
				
				grplist[depthlist[obj.depth]].erase(obj);
				objlist[priolist[obj.priority]].erase(obj);
			}
			
			deletelist = new Array();
			return;
		}
		
		// --------------------------------//
		// 実行
		// --------------------------------//
		public function run():void
		{
			if(lockflag == false)
			{
				for(var i:int = 0; i < objlist.length; i++)
				{
					objlist[i].forEach(function(obj:ObjBase):void
					{
						obj.main();
					});
				}
			}
			else
			{
				objlist[priolist["PRIO_EFFECT"]].forEach(function(obj:ObjBase):void {obj.main();});
				objlist[priolist["PRIO_SYSTEM"]].forEach(function(obj:ObjBase):void {obj.main();});
			}
			
			update();
			
			return;
		}
		
		// --------------------------------//
		// 描画
		// --------------------------------//
		public function draw(surface:BitmapData):void
		{
			for(var i:int = 0; i < grplist.length; i++)
			{
				grplist[i].forEach(function(obj:ObjBase):void
				{
					if(obj.visible == false) {return;}
					obj.draw(surface);
				});
			}
			
			return;
		}
		/*
		// --------------------------------//
		// 全削除
		// --------------------------------//
		public function removeAll():void
		{
			for(var i:int = 0; i < length; i++)
			{
				for(var j:int = 0; j < objlist[i].length; j++)
				{
					removeMovieClip(objlist[i][j]);
				}
			}
			
			return;
		}
		*/
		// --------------------------------//
		// ロック
		// --------------------------------//
		public function lock():void
		{
			lockflag = true;
			return;
		}
		
		public function unlock():void
		{
			lockflag = false;
			return;
		}
		
		// --------------------------------//
		// 指定プライオリティのリストを返す
		// --------------------------------//
		public function searchPriority(prio:String):List
		{
			return objlist[priolist[prio]];
		}
	}
}